/**
 * <copyright> 
 *
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *   Alexander Mark Diewald, fortiss GmbH (diewald@fortiss.org) - Minor adoptions and documentation
 *
 * </copyright>
 *
 * $Id$
 */
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import static org.eclipse.core.runtime.Platform.getBundle;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * {@link CellEditor} for manipulating boolean values within a table. Unlike the
 * {@link CheckboxCellEditor}, this class actually provides a checkbox for manipulating the boolean
 * value, instead of just toggling <code>true</code> and <code>false</code>.
 * 
 * @author diewald
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: EFD0302B4B0E12BEC467B3A359C4DFA7
 */
public class BooleanCellEditor extends CellEditor {

	/** The checkbox button. */
	private Button button;

	/** Default constructor for {@link BooleanCellEditor}. */
	public BooleanCellEditor(Composite parent) {
		super(parent);
	}

	/** Constructs a new {@link BooleanCellEditor} using a given style. */
	public BooleanCellEditor(Composite parent, int style) {
		super(parent, style);
	}

	/** {@inheritDoc} */
	@Override
	protected Control createControl(Composite parent) {
		Font font = parent.getFont();
		Color bg = parent.getBackground();

		button = new Button(parent, getStyle() | SWT.CHECK);
		button.setFont(font);
		button.setBackground(bg);

		return button;
	}

	/** {@inheritDoc} */
	@Override
	protected Object doGetValue() {
		return button.getSelection();
	}

	/** {@inheritDoc} */
	@Override
	protected void doSetValue(Object value) {
		boolean selection = Boolean.TRUE.equals(value);
		button.setSelection(selection);
	}

	/** {@inheritDoc} */
	@Override
	protected void doSetFocus() {
		if(button != null) {
			// Toggle value on first click
			boolean value = Boolean.TRUE.equals(doGetValue());
			doSetValue(!(value));

			// Set focus to button
			button.setFocus();

			// Update view, i.e. remove last visible traces of the icon
			// shown by the corresponding EmulatedNativeCheckBoxLabelProvider
			// (which as already been removed at the time when this method is
			// called).
			button.getParent().layout();

			// For GTK-environments, the layout call is not sufficient to hide
			// the cell editor. Here, it is required to unset the focus. This
			// results in a NPE in windows, so restrict to gtk environments.
			if(getBundle("org.eclipse.swt.gtk.linux.x86_64") != null ||
					getBundle("org.eclipse.swt.gtk.linux.x86") != null) {
				focusLost();
			}

			// TODO (see #2268): This BooleanCellEditor widget is temporarily shown at a wrong
			// position (seen best when holding mouse button down for a longer time
			// when clicking the cell).
		}
	}
}
