/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.compose;

import static org.eclipse.emf.ecore.util.EcoreUtil.delete;
import static org.fortiss.tooling.base.ui.utils.ConstraintsBaseUIUtils.deactivateConfiguration;
import static org.fortiss.tooling.base.utils.ConstraintsBaseUtils.getDefaultConfig;
import static org.fortiss.tooling.base.utils.ConstraintsBaseUtils.isDefaultConfiguration;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.base.model.element.ConstraintConfiguration;
import org.fortiss.tooling.base.model.element.IConstraintBasedProcess;
import org.fortiss.tooling.kernel.extension.IElementCompositor;
import org.fortiss.tooling.kernel.extension.data.IElementCompositionContext;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.model.IProjectRootElement;
import org.fortiss.tooling.kernel.model.constraints.IConstraintInstanceContainer;

/**
 * {@link IElementCompositor} implementation for the {@link IProjectRootElement} s.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 72BD42CADE6B134E646C380FAA6F4C6A
 */
public class ConstraintBasedProcessCompositor<CBP extends IConstraintBasedProcess> implements
		IElementCompositor<CBP> {

	/** {@inheritDoc} */
	@Override
	public boolean canCompose(CBP container, EObject contained, IElementCompositionContext context) {
		return contained instanceof ConstraintConfiguration;
	}

	/** {@inheritDoc} */
	@Override
	public boolean compose(CBP container, EObject contained, IElementCompositionContext context) {
		ConstraintConfiguration config = (ConstraintConfiguration)contained;
		Stream<ConstraintConfiguration> configNamesStream = container.getConfigurations().stream();
		List<String> existingNames =
				configNamesStream.map(c -> c.getName()).collect(Collectors.toList());
		String name = config.getName();
		while(existingNames.contains(name)) {
			name += " bis";
		}
		config.setName(name);
		container.getConfigurations().add(config);
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public boolean canComposePrototype(Prototype prototype) {
		// As of now constraint configurations should never be added by prototype.
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean canDecompose(EObject contained) {
		return contained instanceof ConstraintConfiguration &&
				!isDefaultConfiguration((ConstraintConfiguration)contained);
	}

	/** {@inheritDoc} */
	@Override
	public boolean decompose(EObject contained) {
		if(contained.eContainer() instanceof IConstraintBasedProcess) {
			IConstraintBasedProcess cbdp = (IConstraintBasedProcess)contained.eContainer();
			if(contained.equals(cbdp.getCurrentObjective())) {
				ConstraintConfiguration defaultConfig = getDefaultConfig(cbdp);
				cbdp.setCurrentObjective(defaultConfig);
				if(cbdp.eContainer() instanceof IConstraintInstanceContainer) {
					IConstraintInstanceContainer cstrContainer =
							(IConstraintInstanceContainer)cbdp.eContainer();
					deactivateConfiguration((ConstraintConfiguration)contained, cstrContainer,
							defaultConfig);
				}
			}
		}
		delete(contained);
		return true;
	}
}
