/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.policy;

import static org.fortiss.tooling.base.ui.utils.DragAndDropBaseUtils.createConnectionDragContext;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.fortiss.tooling.base.dnd.AllocationConnectionDragContext;
import org.fortiss.tooling.base.dnd.ConnectionDragContext;
import org.fortiss.tooling.base.ui.editpart.command.AllocationConnectCommand;

/**
 * Connection policy for allocations.
 * 
 * @author eder
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 3FD699A85C20C4AD2D96DBC2B4D3D8ED
 */
public class AllocationDirectConnectionAndReconnectionEditPolicy extends
		DirectConnectionAndReconnectionEditPolicy {

	/** The allocation root object. E.g. a deployment. */
	private EObject allocationRoot;

	/** Constructor. */
	public AllocationDirectConnectionAndReconnectionEditPolicy(EObject allocationRoot) {
		this.allocationRoot = allocationRoot;
	}

	/** {@inheritDoc} */
	@Override
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		ConnectionDragContext context =
				createConnectionDragContext(getHost(), getHost(), request.getLocation(),
						request.getLocation());

		context =
				new AllocationConnectionDragContext(context.getSource(), context.getSourceParent(),
						context.getSourcePosition(), context.getTarget(),
						context.getTargetParent(), context.getTargetPosition(),
						context.isSourceFreeConnector(), context.isTargetFreeConnector(),
						context.isTargetRoot(), allocationRoot);

		AllocationConnectCommand cmd = new AllocationConnectCommand(context);
		request.setStartCommand(cmd);
		return cmd;
	}
}
