/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.model.layout.impl;

import org.fortiss.tooling.base.model.layout.Dimension;
import org.fortiss.tooling.base.model.layout.Point;
import org.fortiss.tooling.base.utils.LayoutModelElementFactory;

/**
 * Static implementation for the {@link Point} layout element.
 * 
 * @author diewald
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 0AD365D763CB75E0F1DA3AFCD313BBE2
 */
public class PointStaticImpl {

	/** Static implementation of the method {@link Point#getDifference(Point)}. */
	public static Dimension getDifference(Point selfPoint, Point compPoint) {
		return LayoutModelElementFactory.createDimension(selfPoint.getX() - compPoint.getX(),
				selfPoint.getY() - compPoint.getY(), "Dim_" + selfPoint.toString() + "<->" +
						compPoint.toString());
	}

	/** Static implementation of the method {@link Point#getTranslated(int, int)}. */
	public static Point getTranslated(Point point, int offsetX, int offsetY) {
		Point retPoint =
				LayoutModelElementFactory.createPoint(point.getX(), point.getY(), point.toString() +
						"_translated");
		return translate(retPoint, offsetX, offsetY);
	}

	/** Static implementation of the method {@link Point#scale(double)}. */
	public static Point scale(Point point, double factor) {
		return scale(point, factor, factor);
	}

	/** Static helper method performing the scale operation. */
	private static Point scale(Point point, double factorX, double factorY) {
		point.setX((int)Math.floor(point.getX() * factorX));
		point.setY((int)Math.floor(point.getY() * factorY));
		return point;
	}

	/** Static implementation of the method {@link Point#translate(int, int)}. */
	public static Point translate(Point point, int offsetX, int offsetY) {
		point.setX(point.getX() + offsetX);
		point.setY(point.getY() + offsetY);
		return point;
	}
}
