/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package test.org.fortiss.tooling.graphicsGL.ui;

import static org.eclipse.swt.SWT.ARROW_DOWN;
import static org.eclipse.swt.SWT.ARROW_LEFT;
import static org.eclipse.swt.SWT.ARROW_RIGHT;
import static org.eclipse.swt.SWT.ARROW_UP;
import static org.eclipse.swt.SWT.PAGE_DOWN;
import static org.eclipse.swt.SWT.PAGE_UP;
import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.enableColorMaterial;
import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.enableLighting;
import static org.fortiss.tooling.graphicsGL.util.Graph3DModelElementFactory.axis;
import static org.fortiss.tooling.graphicsGL.util.Graph3DModelElementFactory.coordSystem;
import static org.fortiss.tooling.graphicsGL.util.Graph3DModelElementFactory.graph;
import static org.fortiss.tooling.graphicsGL.util.Graph3DModelElementFactory.graphLine;
import static org.fortiss.tooling.graphicsGL.util.Graph3DModelElementFactory.graphPoint;
import static org.fortiss.tooling.graphicsGL.util.Graphics3DModelElementFactory.BLUE;
import static org.fortiss.tooling.graphicsGL.util.Graphics3DModelElementFactory.GREEN;
import static org.fortiss.tooling.graphicsGL.util.Graphics3DModelElementFactory.RED;
import static org.fortiss.tooling.graphicsGL.util.Graphics3DModelElementFactory.WHITE;
import static org.fortiss.tooling.graphicsGL.util.Graphics3DModelElementFactory.color;
import static org.fortiss.tooling.graphicsGL.util.Scene3DModelElementFactory.light3D;
import static org.lwjgl.opengl.GL11.GL_LIGHT0;
import static org.lwjgl.opengl.GL11.GL_LIGHT1;
import static org.lwjgl.opengl.GL11.GL_LIGHT2;
import static org.lwjgl.opengl.GL11.GL_LIGHT3;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseEvent;
import org.fortiss.tooling.graphicsGL.model.Color3D;
import org.fortiss.tooling.graphicsGL.model.graph.CoordinateSystem3D;
import org.fortiss.tooling.graphicsGL.model.graph.Graph3D;
import org.fortiss.tooling.graphicsGL.model.graph.GraphLine3D;
import org.fortiss.tooling.graphicsGL.model.graph.GraphPoint3D;
import org.fortiss.tooling.graphicsGL.ui.camera.Camera;
import org.fortiss.tooling.graphicsGL.ui.camera.MovingCameraWithPitchAndYaw;
import org.fortiss.tooling.graphicsGL.ui.editor.Graphical3DEditorBase;
import org.fortiss.tooling.graphicsGL.ui.light.Light3DLight;
import org.fortiss.tooling.graphicsGL.ui.objects.Graph3DViewObject;
import org.fortiss.tooling.graphicsGL.ui.objects.ViewObjectBase;
import org.fortiss.tooling.kernel.model.IProjectRootElement;

/**
 * Test editor for 3d canvas display.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: D873B2128AD79E48A3030E94BAEBFCF1
 */
public class Test3DEditor extends Graphical3DEditorBase<IProjectRootElement> {
	/** The ambient intensity. */
	private static final Color3D AMBIENT_INTENSITY = color(0.8f, 0.8f, 0.8f);

	/** The axis length. */
	private static final float AXIS_LENGTH = 20.0f;

	/** Origin light. */
	private Light3DLight oLight = new Light3DLight(GL_LIGHT0, light3D(0, 0, 0, WHITE));

	/** X axis light. */
	private Light3DLight xLight = new Light3DLight(GL_LIGHT1, light3D(AXIS_LENGTH, 0, 0, RED));

	/** Y axis light. */
	private Light3DLight yLight = new Light3DLight(GL_LIGHT2, light3D(0, AXIS_LENGTH, 0, GREEN));

	/** Z axis light. */
	private Light3DLight zLight = new Light3DLight(GL_LIGHT3, light3D(0, 0, AXIS_LENGTH, BLUE));

	/** The camera. */
	private MovingCameraWithPitchAndYaw camera = new MovingCameraWithPitchAndYaw();

	/** The graph model. */
	private Graph3D graphModel;

	/** The graph display. */
	private Graph3DViewObject graphDisplay;

	/** Constructor. */
	public Test3DEditor() {
		setupTestData();
		graphDisplay = new Graph3DViewObject(graphModel);
		camera.setPosition(1.0f, 1.0f, 1.0f);
	}

	/** {@inheritDoc} */
	@Override
	protected void beginRenderLoop() {
		enableLighting(AMBIENT_INTENSITY);
		oLight.enableLight();
		xLight.enableLight();
		yLight.enableLight();
		zLight.enableLight();
		enableColorMaterial();
	}

	/** {@inheritDoc} */
	@Override
	protected ViewObjectBase getSceneModelObject() {
		return graphDisplay;
	}

	/** {@inheritDoc} */
	@Override
	protected Camera getCamera() {
		return camera;
	}

	/** {@inheritDoc} */
	@Override
	public void mouseDoubleClick(MouseEvent e) {
		// camera.switchLight();
	}

	/** {@inheritDoc} */
	@Override
	public void mouseMove(MouseEvent e) {
		pickX = e.x;
		pickY = e.y;
		// statusBarItem.setText("moved to " + pickX + ":" + pickY);
	}

	/** {@inheritDoc} */
	@Override
	protected void pickTargetChanged() {
		if(pickTarget instanceof GraphPoint3D) {
			GraphPoint3D p = (GraphPoint3D)pickTarget;
			graphDisplay.setHighlightedPoint(p);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void keyReleased(KeyEvent e) {
		float speed = 0.5f;
		switch(e.keyCode) {
			case ARROW_UP:
				camera.forward(speed);
				break;
			case ARROW_DOWN:
				camera.backward(speed);
				break;
			case ARROW_RIGHT:
				camera.right(speed);
				break;
			case ARROW_LEFT:
				camera.left(speed);
				break;
			case PAGE_UP:
				camera.up(speed);
				break;
			case PAGE_DOWN:
				camera.down(speed);
				break;
		}
		switch(e.character) {
			case 'w':
				camera.pitch(15.0f);
				break;
			case 's':
				camera.pitch(-15.0f);
				break;
			case 'a':
				camera.yaw(-15.0f);
				break;
			case 'd':
				camera.yaw(15.0f);
				break;
		}
	}

	/** Setup test data for graph. */
	private void setupTestData() {
		// setup test data
		List<GraphPoint3D> pts = new ArrayList<GraphPoint3D>();
		List<GraphLine3D> lns = new ArrayList<GraphLine3D>();
		GraphPoint3D last = null;
		for(float i = 1; i < 10; i++) {
			GraphPoint3D current = graphPoint(i, i, i > 5 ? 10 - i : i);
			pts.add(current);
			if(last != null) {
				lns.add(graphLine(last, current));
			}
			last = current;
		}
		CoordinateSystem3D sys =
				coordSystem(axis(AXIS_LENGTH, RED), axis(AXIS_LENGTH, GREEN),
						axis(AXIS_LENGTH, BLUE));
		graphModel = graph(sys, pts, lns);
	}
}
