/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection;

import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;

/**
 * Class built with WindowBuilder => no comments
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: AF4EC47B20F1F701D6560C8119CB5549
 */
public class KISSViewerGUI extends Composite {
	private TreeViewer treeViewer;
	private ScrolledComposite detailsScrolledComposite;
	private Composite descriptionComposite;
	private Text descriptionText;

	/**
	 * Create the composite.
	 * 
	 * @param parent
	 * @param style
	 */
	public KISSViewerGUI(Composite parent, int style) {
		super(parent, style);
		setLayout(new FillLayout(SWT.HORIZONTAL));

		SashForm verticalSplitSash = new SashForm(this, SWT.VERTICAL | SWT.BORDER | SWT.SMOOTH);

		SashForm horizontalSplitSash = new SashForm(verticalSplitSash, SWT.BORDER | SWT.SMOOTH);

		treeViewer = new TreeViewer(horizontalSplitSash, SWT.SINGLE | SWT.BORDER);
		treeViewer.setAutoExpandLevel(2);
		Tree tree = treeViewer.getTree();

		detailsScrolledComposite =
				new ScrolledComposite(horizontalSplitSash, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		detailsScrolledComposite.setExpandHorizontal(true);
		detailsScrolledComposite.setExpandVertical(true);
		horizontalSplitSash.setWeights(new int[] {1, 5});

		descriptionComposite = new Composite(verticalSplitSash, SWT.BORDER);
		descriptionComposite.setLayout(new GridLayout());
		verticalSplitSash.setWeights(new int[] {7, 1});

		descriptionText = new Text(descriptionComposite, SWT.READ_ONLY | SWT.V_SCROLL | SWT.MULTI);
		Color gray = Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);
		descriptionText.setBackground(gray);
		descriptionText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	}

	public TreeViewer getTreeViewer() {
		return treeViewer;
	}

	public ScrolledComposite getDetailsScrolledComposite() {
		return detailsScrolledComposite;
	}

	public Composite getDescriptionComposite() {
		return descriptionComposite;
	}

	public Text getDescriptionText() {
		return descriptionText;
	}
}
