/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.util;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.part.MultiPageEditorSite;
import org.fortiss.tooling.kernel.ui.extension.IModelEditor;
import org.fortiss.tooling.kernel.ui.internal.editor.ExtendableMultiPageEditor;
import org.fortiss.tooling.kernel.ui.service.IModelEditorBindingService;

/**
 * Utility methods related to editors.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 7F41C575A0C647A15DF6FCEDE37E4C77
 */
public class EditorUtils {
	/**
	 * @param editorClazz
	 *            Class of the editor you would like to get.
	 * @param activateEditor
	 *            Set to true if you want the editor to be not just returned, but also activated in
	 *            the GUI.
	 * @return the opened editor of the given class if any.
	 */
	@SuppressWarnings("unchecked")
	public static <T extends EObject, E extends IModelEditor<T>> E getOpenedEditor(
			Class<E> editorClazz, boolean activateEditor) {
		IModelEditor<?> editor = IModelEditorBindingService.getInstance().getActiveEditor();
		if(editorClazz.isInstance(editor)) {
			return (E)editor;
		}
		if(editor.getSite() instanceof MultiPageEditorSite) {
			MultiPageEditorSite multiEditor = (MultiPageEditorSite)editor.getSite();
			ExtendableMultiPageEditor multiPage =
					(ExtendableMultiPageEditor)multiEditor.getMultiPageEditor();
			E saEditor = multiPage.getPageEditor(editorClazz);
			if(activateEditor) {
				multiPage.setActiveEditor(saEditor);
			}
			return saEditor;
		}
		return null;
	}
}
