/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension;

import org.fortiss.tooling.kernel.model.constraints.ConstraintInstance;
import org.fortiss.tooling.kernel.model.constraints.IConstrained;
import org.fortiss.tooling.kernel.model.constraints.IConstraintInstanceStatus;
import org.fortiss.tooling.kernel.service.IConstraintService;

/**
 * Interface for constraints as required by {@link IConstraintService}.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 50DDDEC610FF3E8FDFFB13DD99B035FB
 */
public interface IConstraint {

	/** Verify the given constraint instance. */
	IConstraintInstanceStatus verify(ConstraintInstance ci);

	/**
	 * Hook to preprocess <code>cstrd</code> before computing its checksum. Typically useful to
	 * remove items which should not be considered when computing the checksum, e.g., layout
	 * information.
	 */
	void preprocessBeforeChecksum(IConstrained cstrd);

	/** Cancels the last verification of <code>ci</code>. */
	void cancel(ConstraintInstance ci);

	/**
	 * Adds a constraint instance to the provided element.
	 * 
	 * Subclasses shall return <code>null</code> if the provided element should not be constrained
	 * (whatever is the reason: the constraint is only judge here).
	 */
	ConstraintInstance addConstraintInstanceIfNeeded(IConstrained cstrd);
}
