/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.marker;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.conqat.lib.commons.string.StringUtils;

/**
 * An enum for representing task tags.
 * 
 * @author burger
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: A9A6B3E1C18E03C8326D26A3E2F6227F
 */
public enum ETaskTag {

	/** Represents a task tag. */
	TODO,

	/** Represents a task tag. */
	FIXME,

	/** Represents a task tag. */
	BUG;

	/** Returns a regex pattern for the task tags */
	private static Pattern getPatternForTaskTags() {
		// match task tag
		String tags = "(" + StringUtils.concat(ETaskTag.values(), "|") + ")";

		// matches expressions with ,.:;{[- or a space after the task tag
		String comment = "(?:([ ,.:;\\{\\(\\[-].*?)\\s*)?$";

		return Pattern.compile(tags + comment, Pattern.MULTILINE);
	}

	/**
	 * Searches for tasks in a (multiline) comment string and returns them.
	 */
	public static String[] extractTaskTags(String comment) {
		ArrayList<String> tasks = new ArrayList<String>();

		Pattern p = getPatternForTaskTags();
		Matcher m = p.matcher(comment);

		while (m.find()) {
			String task = m.group(1);
			if (m.group(2) != null) {
				task += m.group(2);
			}
			tasks.add(task);
		}

		return tasks.toArray(new String[] {});
	}
}